 ; Copyright 2006-2018 Intel Corporation
 ;
 ; This software and the related documents are Intel copyrighted materials, and
 ; your use of them is governed by the express license under which they were
 ; provided to you (License). Unless the License provides otherwise, you may
 ; not use, modify, copy, publish, distribute, disclose, or transmit this
 ; software or the related documents without Intel's prior written permission.
 ;
 ; This software and the related documents are provided as is, with no express
 ; or implied warranties, other than those that are expressly stated in the
 ; License.

(defclass SSF
  "This class represents the data collected from the ssf_version file.
   Instances of the class are created, and the data slots are populated,
   by the analyze extension component."
  (is-a BASE_NODE TIMESTAMP DATABASE)
  (role concrete)
  (pattern-match reactive)

  (multislot ssf-version (type STRING)) ; SSF Version 
  (multislot ssf-layer (type STRING)) ; SSF layer 
  (slot stderr (type STRING))) ; STDERR string

(defmessage-handler SSF maximum-version ()
  "Returns the maximum version contained in the ssf-version slot."
  (bind ?maxver (nth$ 1 ?self:ssf-version))
  (progn$ (?ver ?self:ssf-version)
          (if (= (vercmp ?ver ?maxver) 1) then (bind ?maxver ?ver)))
  (return ?maxver))

(defmessage-handler SSF minimum-version ()
  "Returns the maximum version contained in the ssf-version slot."
  (bind ?minver (nth$ 1 ?self:ssf-version))
  (progn$ (?ver ?self:ssf-version)
          (if (= (vercmp ?ver ?minver) -1) then (bind ?minver ?ver)))
  (return ?minver))

(defmessage-handler SSF get-version (?layer)
  (bind ?i (member$ ?layer ?self:ssf-layer))
  (return (nth$ ?i ?self:ssf-version)))
