#!/bin/bash

# Copyright (C) 2017-2018 Intel Corporation. All Rights Reserved.
#
# The  source code contained or described  herein and all documents  related to
# the source code ('Material') are owned by Intel  Corporation or its suppliers
# or licensors.  Title to the Material  remains  with Intel  Corporation or its
# suppliers and licensors. The Material is protected by worldwide copyright and
# trade secret laws and treaty provisions. No part of the Material may be used,
# copied,  reproduced,  modified,  published,  uploaded,  posted,  transmitted,
# distributed,  or disclosed in any way without  Intel's prior express  written
# permission.
#
# No license under any patent,  copyright,  trade secret or other  intellectual
# property  right is granted to or conferred upon you by disclosure or delivery
# of the Materials,  either expressly, by implication,  inducement, estoppel or
# otherwise.  Any license  under  such  intellectual  property  rights  must be
# express and approved by Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or alter this
# notice  or  any  other  notice  embedded  in  Materials  by Intel or  Intel's
# suppliers or licensors in any way.

. $(dirname $0)/../../functions

# check script parameteres to make sure nothing is critically wrong
function check {
    if [ ! -f $1 ]; then
        \echo "\"$1\" does not exist" >&2
        exit 1
    elif [[ "$1" == "" ]]; then
        \echo "file \"$2\" not provided" >&2
        exit 2
    elif [[ ! "$1" =~ "$2" ]]; then
        \echo "file \"$1\" may not be used for file \"$2\"" >&2
        exit 3
    else
        return 0
    fi
}


# INPUT: filename
# OUTPUT: filename,perms,user,group,md5sum
function file_info {

    if \stat $1 &>/dev/null; then
        P=$(\stat --printf='%A' $1)
        O=$(\stat --printf='%U' $1)
        G=$(\stat --printf='%G' $1)
        STAT_AVAILABLE=1
    else
        P=""
        O=""
        G=""
        STAT_AVAILABLE=0
    fi

    if \md5sum $1 &>/dev/null; then
        M=$(\md5sum $1 | \awk '{print $1}')
        MD5SUM_AVAILABLE=1
    else
        M=""
        MD5SUM_AVAILABLE=0
    fi

    # file, perms, owner, group, md5sum
    OUTPUT="$1,$P,$O,$G,$M"
    if [ $STAT_AVAILABLE -eq 1 ] && [ $MD5SUM_AVAILABLE -eq 1 ]; then
        \echo $OUTPUT >&1
    else
        _stderr $OUTPUT
    fi

}


# main function
function main {

    check $1 "/white";  white=$1
    check $2 "/black";  black=$2

    # blacklists are interpreted as path regular expressions
    for file in $(\rpm --query --all --configfiles | \grep -v "^[^/]" | \
      \grep -vGf $black); do
        file_info $file
    done

    # whitelists are interpreted as absolute file paths
    while read file; do
        file_info $file
    done < $white

}


main $@
